@@ -13,7 +13,9 @@
  * from the use of this software.
  */
 
+#include <libpkgconf/stdinc.h>
 #include <libpkgconf/libpkgconf.h>
+#include <libpkgconf/config.h>
 
 /*
  * !doc
@@ -137,6 +139,9 @@ pkgconf_client_deinit(pkgconf_client_t *client)
 	if (client->buildroot_dir != NULL)
 		free(client->buildroot_dir);
 
+	pkgconf_path_free(&client->filter_libdirs);
+	pkgconf_path_free(&client->filter_includedirs);
+
 	pkgconf_tuple_free_global(client);
 	pkgconf_path_free(&client->dir_list);
 	pkgconf_cache_free(client);
@@ -319,7 +324,10 @@ pkgconf_trace(const pkgconf_client_t *client, const char *filename, size_t linen
 	size_t len;
 	va_list va;
 
-	len = snprintf(errbuf, sizeof errbuf, "%s:%zu [%s]: ", filename, lineno, funcname);
+	if (client == NULL)
+		return false;
+
+	len = snprintf(errbuf, sizeof errbuf, "%s:" SIZE_FMT_SPECIFIER " [%s]: ", filename, lineno, funcname);
 
 	va_start(va, format);
 	vsnprintf(errbuf + len, sizeof(errbuf) - len, format, va);
