@@ -13,6 +13,7 @@
  * from the use of this software.
  */
 
+#include <libpkgconf/stdinc.h>
 #include <libpkgconf/config.h>
 #include <libpkgconf/libpkgconf.h>
 
@@ -30,10 +31,11 @@
 #	define PKG_CONFIG_REG_KEY "Software\\pkgconfig\\PKG_CONFIG_PATH"
 #	undef PKG_DEFAULT_PATH
 #	define PKG_DEFAULT_PATH "../lib/pkgconfig;../share/pkgconfig"
+#	define strncasecmp _strnicmp
+#	define strcasecmp _stricmp
 #endif
 
-#define PKG_CONFIG_EXT		".pc"
-#define PKG_CONFIG_PATH_SZ	(65535)
+#define PKG_CONFIG_EXT ".pc"
 
 static inline bool
 str_has_suffix(const char *str, const char *suffix)
@@ -48,10 +50,9 @@ str_has_suffix(const char *str, const char *suffix)
 }
 
 static inline const char *
-get_default_pkgconfig_path(void)
+get_default_pkgconfig_path(char *outbuf, size_t outlen)
 {
 #ifdef _WIN32
-	static char outbuf[MAX_PATH];
 	char namebuf[MAX_PATH];
 	char *p;
 
@@ -67,27 +68,29 @@ get_default_pkgconfig_path(void)
 		return PKG_DEFAULT_PATH;
 
 	*p = '\0';
-	pkgconf_strlcpy(outbuf, namebuf, sizeof outbuf);
-	pkgconf_strlcat(outbuf, "/", sizeof outbuf);
-	pkgconf_strlcat(outbuf, "../lib/pkgconfig", sizeof outbuf);
-	pkgconf_strlcat(outbuf, ";", sizeof outbuf);
-	pkgconf_strlcat(outbuf, namebuf, sizeof outbuf);
-	pkgconf_strlcat(outbuf, "/", sizeof outbuf);
-	pkgconf_strlcat(outbuf, "../share/pkgconfig", sizeof outbuf);
+	pkgconf_strlcpy(outbuf, namebuf, outlen);
+	pkgconf_strlcat(outbuf, "/", outlen);
+	pkgconf_strlcat(outbuf, "../lib/pkgconfig", outlen);
+	pkgconf_strlcat(outbuf, ";", outlen);
+	pkgconf_strlcat(outbuf, namebuf, outlen);
+	pkgconf_strlcat(outbuf, "/", outlen);
+	pkgconf_strlcat(outbuf, "../share/pkgconfig", outlen);
 
 	return outbuf;
+#else
+	(void) outbuf;
+	(void) outlen;
 #endif
 
 	return PKG_DEFAULT_PATH;
 }
 
 static const char *
-pkg_get_parent_dir(pkgconf_pkg_t *pkg)
+pkg_get_parent_dir(pkgconf_pkg_t *pkg, char *buf, size_t buflen)
 {
-	static char buf[PKGCONF_BUFSIZE];
 	char *pathbuf;
 
-	pkgconf_strlcpy(buf, pkg->filename, sizeof buf);
+	pkgconf_strlcpy(buf, pkg->filename, buflen);
 	pathbuf = strrchr(buf, PKG_DIR_SEP_S);
 	if (pathbuf == NULL)
 		pathbuf = strrchr(buf, '/');
@@ -115,10 +118,14 @@ pkgconf_pkg_dir_list_build(pkgconf_client_t *client)
 	pkgconf_path_build_from_environ("PKG_CONFIG_PATH", NULL, &client->dir_list, true);
 
 	if (!(client->flags & PKGCONF_PKG_PKGF_ENV_ONLY))
-		pkgconf_path_build_from_environ("PKG_CONFIG_LIBDIR", get_default_pkgconfig_path(), &client->dir_list, true);
+	{
+		char pathbuf[PKGCONF_BUFSIZE];
+
+		pkgconf_path_build_from_environ("PKG_CONFIG_LIBDIR", get_default_pkgconfig_path(pathbuf, sizeof pathbuf), &client->dir_list, true);
+	}
 }
 
-typedef void (*pkgconf_pkg_parser_keyword_func_t)(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const ptrdiff_t offset, char *value);
+typedef void (*pkgconf_pkg_parser_keyword_func_t)(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *keyword, const size_t lineno, const ptrdiff_t offset, char *value);
 typedef struct {
 	const char *keyword;
 	const pkgconf_pkg_parser_keyword_func_t func;
@@ -132,23 +139,35 @@ static int pkgconf_pkg_parser_keyword_pair_cmp(const void *key, const void *ptr)
 }
 
 static void
-pkgconf_pkg_parser_tuple_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const ptrdiff_t offset, char *value)
+pkgconf_pkg_parser_tuple_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *keyword, const size_t lineno, const ptrdiff_t offset, char *value)
 {
-	char **dest = ((void *) pkg + offset);
+	(void) keyword;
+	(void) lineno;
+
+	char **dest = (char **)((char *) pkg + offset);
 	*dest = pkgconf_tuple_parse(client, &pkg->vars, value);
 }
 
 static void
-pkgconf_pkg_parser_fragment_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const ptrdiff_t offset, char *value)
+pkgconf_pkg_parser_fragment_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *keyword, const size_t lineno, const ptrdiff_t offset, char *value)
 {
-	pkgconf_list_t *dest = ((void *) pkg + offset);
-	pkgconf_fragment_parse(client, dest, &pkg->vars, value);
+	pkgconf_list_t *dest = (pkgconf_list_t *)((char *) pkg + offset);
+	bool ret = pkgconf_fragment_parse(client, dest, &pkg->vars, value);
+
+	if (!ret)
+	{
+		pkgconf_warn(client, "%s:" SIZE_FMT_SPECIFIER ": warning: unable to parse field '%s' into an argument vector, value [%s]\n", pkg->filename,
+			     lineno, keyword, value);
+	}
 }
 
 static void
-pkgconf_pkg_parser_dependency_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const ptrdiff_t offset, char *value)
+pkgconf_pkg_parser_dependency_func(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *keyword, const size_t lineno, const ptrdiff_t offset, char *value)
 {
-	pkgconf_list_t *dest = ((void *) pkg + offset);
+	(void) keyword;
+	(void) lineno;
+
+	pkgconf_list_t *dest = (pkgconf_list_t *)((char *) pkg + offset);
 	pkgconf_dependency_parse(client, pkg, dest, value);
 }
 
@@ -162,13 +181,13 @@ static const pkgconf_pkg_parser_keyword_pair_t pkgconf_pkg_parser_keyword_funcs[
 	{"LIBS.private", pkgconf_pkg_parser_fragment_func, offsetof(pkgconf_pkg_t, libs_private)},
 	{"Name", pkgconf_pkg_parser_tuple_func, offsetof(pkgconf_pkg_t, realname)},
 	{"Provides", pkgconf_pkg_parser_dependency_func, offsetof(pkgconf_pkg_t, provides)},
-	{"Requires", pkgconf_pkg_parser_dependency_func, offsetof(pkgconf_pkg_t, requires)},
+	{"Requires", pkgconf_pkg_parser_dependency_func, offsetof(pkgconf_pkg_t, required)},
 	{"Requires.private", pkgconf_pkg_parser_dependency_func, offsetof(pkgconf_pkg_t, requires_private)},
 	{"Version", pkgconf_pkg_parser_tuple_func, offsetof(pkgconf_pkg_t, version)},
 };
 
 static bool
-pkgconf_pkg_parser_keyword_set(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const char *keyword, char *value)
+pkgconf_pkg_parser_keyword_set(const pkgconf_client_t *client, pkgconf_pkg_t *pkg, const size_t lineno, const char *keyword, char *value)
 {
 	const pkgconf_pkg_parser_keyword_pair_t *pair = bsearch(keyword,
 		pkgconf_pkg_parser_keyword_funcs, PKGCONF_ARRAY_SIZE(pkgconf_pkg_parser_keyword_funcs),
@@ -177,18 +196,17 @@ pkgconf_pkg_parser_keyword_set(const pkgconf_client_t *client, pkgconf_pkg_t *pk
 	if (pair == NULL || pair->func == NULL)
 		return false;
 
-	pair->func(client, pkg, pair->offset, value);
+	pair->func(client, pkg, keyword, lineno, pair->offset, value);
 	return true;
 }
 
 static const char *
-determine_prefix(const pkgconf_pkg_t *pkg)
+determine_prefix(const pkgconf_pkg_t *pkg, char *buf, size_t buflen)
 {
-	static char buf[PKGCONF_BUFSIZE];
 	char *pathiter;
 
-	pkgconf_strlcpy(buf, pkg->filename, sizeof buf);
-	pkgconf_path_relocate(buf, sizeof buf);
+	pkgconf_strlcpy(buf, pkg->filename, buflen);
+	pkgconf_path_relocate(buf, buflen);
 
 	pathiter = strrchr(buf, PKG_DIR_SEP_S);
 	if (pathiter == NULL)
@@ -238,7 +256,7 @@ pkgconf_pkg_validate(const pkgconf_client_t *client, const pkgconf_pkg_t *pkg)
 
 	for (i = 0; i < PKGCONF_ARRAY_SIZE(pkgconf_pkg_validations); i++)
 	{
-		char **p = ((void *) pkg + pkgconf_pkg_validations[i].offset);
+		char **p = (char **)((char *) pkg + pkgconf_pkg_validations[i].offset);
 
 		if (*p != NULL)
 			continue;
@@ -268,16 +286,21 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 {
 	pkgconf_pkg_t *pkg;
 	char readbuf[PKGCONF_BUFSIZE];
+	char pathbuf[PKGCONF_ITEM_SIZE];
 	char *idptr;
 	size_t lineno = 0;
 
 	pkg = calloc(sizeof(pkgconf_pkg_t), 1);
 	pkg->filename = strdup(filename);
-	pkgconf_tuple_add(client, &pkg->vars, "pcfiledir", pkg_get_parent_dir(pkg), true);
+	pkgconf_tuple_add(client, &pkg->vars, "pcfiledir", pkg_get_parent_dir(pkg, pathbuf, sizeof pathbuf), true);
 
 	/* make module id */
 	if ((idptr = strrchr(pkg->filename, PKG_DIR_SEP_S)) != NULL)
 		idptr++;
+#ifdef _WIN32
+	else if ((idptr = strrchr(pkg->filename, '/')) != NULL)
+		idptr++;
+#endif
 	else
 		idptr = pkg->filename;
 
@@ -293,7 +316,7 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 
 		lineno++;
 
-		PKGCONF_TRACE(client, "%s:%zu > [%s]", filename, lineno, readbuf);
+		PKGCONF_TRACE(client, "%s:" SIZE_FMT_SPECIFIER " > [%s]", filename, lineno, readbuf);
 
 		p = readbuf;
 		while (*p && (isalpha((unsigned int)*p) || isdigit((unsigned int)*p) || *p == '_' || *p == '.'))
@@ -307,7 +330,7 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 		{
 			if (!warned_key_whitespace)
 			{
-				pkgconf_warn(client, "%s:%zu: warning: whitespace encountered while parsing key section\n",
+				pkgconf_warn(client, "%s:" SIZE_FMT_SPECIFIER ": warning: whitespace encountered while parsing key section\n",
 					pkg->filename, lineno);
 				warned_key_whitespace = true;
 			}
@@ -330,7 +353,7 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 		{
 			if (!warned_value_whitespace && op == '=')
 			{
-				pkgconf_warn(client, "%s:%zu: warning: trailing whitespace encountered while parsing value section\n",
+				pkgconf_warn(client, "%s:" SIZE_FMT_SPECIFIER ": warning: trailing whitespace encountered while parsing value section\n",
 					pkg->filename, lineno);
 				warned_value_whitespace = true;
 			}
@@ -342,14 +365,14 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 		switch (op)
 		{
 		case ':':
-			pkgconf_pkg_parser_keyword_set(client, pkg, key, value);
+			pkgconf_pkg_parser_keyword_set(client, pkg, lineno, key, value);
 			break;
 		case '=':
 			if (strcmp(key, client->prefix_varname) || !(client->flags & PKGCONF_PKG_PKGF_REDEFINE_PREFIX))
 				pkgconf_tuple_add(client, &pkg->vars, key, value, true);
 			else
 			{
-				const char *relvalue = determine_prefix(pkg);
+				const char *relvalue = determine_prefix(pkg, pathbuf, sizeof pathbuf);
 				if (relvalue != NULL)
 				{
 					pkgconf_tuple_add(client, &pkg->vars, "orig_prefix", value, true);
@@ -392,12 +415,15 @@ pkgconf_pkg_new_from_file(pkgconf_client_t *client, const char *filename, FILE *
 void
 pkgconf_pkg_free(pkgconf_client_t *client, pkgconf_pkg_t *pkg)
 {
-	if (pkg == NULL || pkg->flags & PKGCONF_PKG_PROPF_STATIC)
+	if (pkg == NULL)
+		return;
+
+	if (pkg->flags & PKGCONF_PKG_PROPF_STATIC && !(pkg->flags & PKGCONF_PKG_PROPF_VIRTUAL))
 		return;
 
 	pkgconf_cache_remove(client, pkg);
 
-	pkgconf_dependency_free(&pkg->requires);
+	pkgconf_dependency_free(&pkg->required);
 	pkgconf_dependency_free(&pkg->requires_private);
 	pkgconf_dependency_free(&pkg->conflicts);
 	pkgconf_dependency_free(&pkg->provides);
@@ -409,6 +435,9 @@ pkgconf_pkg_free(pkgconf_client_t *client, pkgconf_pkg_t *pkg)
 
 	pkgconf_tuple_free(&pkg->vars);
 
+	if (pkg->flags & PKGCONF_PKG_PROPF_VIRTUAL)
+		return;
+
 	if (pkg->id != NULL)
 		free(pkg->id);
 
@@ -446,11 +475,15 @@ pkgconf_pkg_free(pkgconf_client_t *client, pkgconf_pkg_t *pkg)
  *    :rtype: pkgconf_pkg_t *
  */
 pkgconf_pkg_t *
-pkgconf_pkg_ref(const pkgconf_client_t *client, pkgconf_pkg_t *pkg)
+pkgconf_pkg_ref(pkgconf_client_t *client, pkgconf_pkg_t *pkg)
 {
-	(void) client;
+	if (pkg->owner != NULL && pkg->owner != client)
+		PKGCONF_TRACE(client, "WTF: client %p refers to package %p owned by other client %p", client, pkg, pkg->owner);
 
+	pkg->owner = client;
 	pkg->refcount++;
+	PKGCONF_TRACE(client, "refcount@%p: %d", pkg, pkg->refcount);
+
 	return pkg;
 }
 
@@ -468,9 +501,14 @@ pkgconf_pkg_ref(const pkgconf_client_t *client, pkgconf_pkg_t *pkg)
 void
 pkgconf_pkg_unref(pkgconf_client_t *client, pkgconf_pkg_t *pkg)
 {
+	if (pkg->owner != NULL && pkg->owner != client)
+		PKGCONF_TRACE(client, "WTF: client %p unrefs package %p owned by other client %p", client, pkg, pkg->owner);
+
 	pkg->refcount--;
+	PKGCONF_TRACE(pkg->owner, "refcount@%p: %d", pkg, pkg->refcount);
+
 	if (pkg->refcount <= 0)
-		pkgconf_pkg_free(client, pkg);
+		pkgconf_pkg_free(pkg->owner, pkg);
 }
 
 static inline pkgconf_pkg_t *
@@ -478,8 +516,8 @@ pkgconf_pkg_try_specific_path(pkgconf_client_t *client, const char *path, const
 {
 	pkgconf_pkg_t *pkg = NULL;
 	FILE *f;
-	char locbuf[PKG_CONFIG_PATH_SZ];
-	char uninst_locbuf[PKG_CONFIG_PATH_SZ];
+	char locbuf[PKGCONF_ITEM_SIZE];
+	char uninst_locbuf[PKGCONF_ITEM_SIZE];
 
 	PKGCONF_TRACE(client, "trying path: %s for %s", path, name);
 
@@ -516,7 +554,7 @@ pkgconf_pkg_scan_dir(pkgconf_client_t *client, const char *path, void *data, pkg
 
 	for (dirent = readdir(dir); dirent != NULL; dirent = readdir(dir))
 	{
-		static char filebuf[PKGCONF_BUFSIZE];
+		char filebuf[PKGCONF_ITEM_SIZE];
 		pkgconf_pkg_t *pkg;
 		FILE *f;
 
@@ -587,7 +625,7 @@ pkgconf_scan_all(pkgconf_client_t *client, void *data, pkgconf_pkg_iteration_fun
 
 #ifdef _WIN32
 static pkgconf_pkg_t *
-pkgconf_pkg_find_in_registry_key(const pkgconf_client_t *client, HKEY hkey, const char *name)
+pkgconf_pkg_find_in_registry_key(pkgconf_client_t *client, HKEY hkey, const char *name)
 {
 	pkgconf_pkg_t *pkg = NULL;
 
@@ -603,7 +641,7 @@ pkgconf_pkg_find_in_registry_key(const pkgconf_client_t *client, HKEY hkey, cons
 	while (RegEnumValue(key, i++, buf, &bufsize, NULL, NULL, NULL, NULL)
 			== ERROR_SUCCESS)
 	{
-		char pathbuf[PKG_CONFIG_PATH_SZ];
+		char pathbuf[PKGCONF_ITEM_SIZE];
 		DWORD type;
 		DWORD pathbuflen = sizeof pathbuf;
 
@@ -638,6 +676,7 @@ pkgconf_pkg_find_in_registry_key(const pkgconf_client_t *client, HKEY hkey, cons
 pkgconf_pkg_t *
 pkgconf_pkg_find(pkgconf_client_t *client, const char *name)
 {
+	char pathbuf[PKGCONF_ITEM_SIZE];
 	pkgconf_pkg_t *pkg = NULL;
 	pkgconf_node_t *n;
 	FILE *f;
@@ -656,7 +695,7 @@ pkgconf_pkg_find(pkgconf_client_t *client, const char *name)
 			pkg = pkgconf_pkg_new_from_file(client, name, f);
 			if (pkg != NULL)
 			{
-				pkgconf_path_add(pkg_get_parent_dir(pkg), &client->dir_list, true);
+				pkgconf_path_add(pkg_get_parent_dir(pkg, pathbuf, sizeof pathbuf), &client->dir_list, true);
 				return pkg;
 			}
 		}
@@ -675,8 +714,6 @@ pkgconf_pkg_find(pkgconf_client_t *client, const char *name)
 		if ((pkg = pkgconf_cache_lookup(client, name)) != NULL)
 		{
 			PKGCONF_TRACE(client, "%s is cached", name);
-
-			pkg->flags |= PKGCONF_PKG_PROPF_CACHED;
 			return pkg;
 		}
 	}
@@ -719,7 +756,7 @@ int
 pkgconf_compare_version(const char *a, const char *b)
 {
 	char oldch1, oldch2;
-	char buf1[PKGCONF_BUFSIZE], buf2[PKGCONF_BUFSIZE];
+	char buf1[PKGCONF_ITEM_SIZE], buf2[PKGCONF_ITEM_SIZE];
 	char *str1, *str2;
 	char *one, *two;
 	int ret;
@@ -847,15 +884,25 @@ static pkgconf_pkg_t pkg_config_virtual = {
 	.flags = PKGCONF_PKG_PROPF_STATIC,
 	.vars = {
 		.head = &(pkgconf_node_t){
-			.prev = NULL,
-			.next = NULL,
+			.next = &(pkgconf_node_t){
+				.next = &(pkgconf_node_t){
+					.data = &(pkgconf_tuple_t){
+						.key = "pc_system_libdirs",
+						.value = SYSTEM_LIBDIR,
+					}
+				},
+				.data = &(pkgconf_tuple_t){
+					.key = "pc_system_includedirs",
+					.value = SYSTEM_INCLUDEDIR,
+				}
+			},
 			.data = &(pkgconf_tuple_t){
 				.key = "pc_path",
 				.value = PKG_DEFAULT_PATH,
 			},
 		},
 		.tail = NULL,
-	},
+	}
 };
 
 static pkgconf_pkg_t pkgconf_virtual = {
@@ -867,8 +914,18 @@ static pkgconf_pkg_t pkgconf_virtual = {
 	.flags = PKGCONF_PKG_PROPF_STATIC,
 	.vars = {
 		.head = &(pkgconf_node_t){
-			.prev = NULL,
-			.next = NULL,
+			.next = &(pkgconf_node_t){
+				.next = &(pkgconf_node_t){
+					.data = &(pkgconf_tuple_t){
+						.key = "pc_system_libdirs",
+						.value = SYSTEM_LIBDIR,
+					}
+				},
+				.data = &(pkgconf_tuple_t){
+					.key = "pc_system_includedirs",
+					.value = SYSTEM_INCLUDEDIR,
+				}
+			},
 			.data = &(pkgconf_tuple_t){
 				.key = "pc_path",
 				.value = PKG_DEFAULT_PATH,
@@ -1048,8 +1105,12 @@ typedef struct {
 
 static const pkgconf_pkg_provides_vermatch_rule_t pkgconf_pkg_provides_vermatch_rules[] = {
 	[PKGCONF_CMP_ANY] = {
-		.rulecmp = {},
-		.depcmp = {},
+		.rulecmp = {
+			[PKGCONF_CMP_ANY]			= pkgconf_pkg_comparator_none,
+                },
+		.depcmp = {
+			[PKGCONF_CMP_ANY]			= pkgconf_pkg_comparator_none,
+                },
 	},
 	[PKGCONF_CMP_LESS_THAN] = {
 		.rulecmp = {
@@ -1121,7 +1182,9 @@ static const pkgconf_pkg_provides_vermatch_rule_t pkgconf_pkg_provides_vermatch_
 			[PKGCONF_CMP_EQUAL]			= pkgconf_pkg_comparator_eq,
 			[PKGCONF_CMP_NOT_EQUAL]			= pkgconf_pkg_comparator_ne
 		},
-		.depcmp = {},
+		.depcmp = {
+			[PKGCONF_CMP_ANY]			= pkgconf_pkg_comparator_none,
+                },
 	},
 	[PKGCONF_CMP_NOT_EQUAL] = {
 		.rulecmp = {
@@ -1133,7 +1196,9 @@ static const pkgconf_pkg_provides_vermatch_rule_t pkgconf_pkg_provides_vermatch_
 			[PKGCONF_CMP_EQUAL]			= pkgconf_pkg_comparator_ne,
 			[PKGCONF_CMP_NOT_EQUAL]			= pkgconf_pkg_comparator_eq
 		},
-		.depcmp = {},
+		.depcmp = {
+			[PKGCONF_CMP_ANY]			= pkgconf_pkg_comparator_none,
+                },
 	},
 };
 
@@ -1196,7 +1261,10 @@ pkgconf_pkg_scan_providers(pkgconf_client_t *client, pkgconf_dependency_t *pkgde
 
 	pkg = pkgconf_scan_all(client, &ctx, (pkgconf_pkg_iteration_func_t) pkgconf_pkg_scan_provides_entry);
 	if (pkg != NULL)
+	{
+		pkgdep->match = pkgconf_pkg_ref(client, pkg);
 		return pkg;
+	}
 
 	if (eflags != NULL)
 		*eflags |= PKGCONF_PKG_ERRF_PACKAGE_NOT_FOUND;
@@ -1228,6 +1296,12 @@ pkgconf_pkg_verify_dependency(pkgconf_client_t *client, pkgconf_dependency_t *pk
 
 	PKGCONF_TRACE(client, "trying to verify dependency: %s", pkgdep->package);
 
+	if (pkgdep->match != NULL)
+	{
+		PKGCONF_TRACE(client, "cached dependency: %s -> %s@%p", pkgdep->package, pkgdep->match->id, pkgdep->match);
+		return pkgconf_pkg_ref(client, pkgdep->match);
+	}
+
 	pkg = pkgconf_pkg_find(client, pkgdep->package);
 	if (pkg == NULL)
 	{
@@ -1245,11 +1319,13 @@ pkgconf_pkg_verify_dependency(pkgconf_client_t *client, pkgconf_dependency_t *pk
 	if (pkg->id == NULL)
 		pkg->id = strdup(pkgdep->package);
 
-	if (pkgconf_pkg_comparator_impls[pkgdep->compare](pkg->version, pkgdep->version) == true)
-		return pkg;
-
-	if (eflags != NULL)
-		*eflags |= PKGCONF_PKG_ERRF_PACKAGE_VER_MISMATCH;
+	if (pkgconf_pkg_comparator_impls[pkgdep->compare](pkg->version, pkgdep->version) != true)
+	{
+		if (eflags != NULL)
+			*eflags |= PKGCONF_PKG_ERRF_PACKAGE_VER_MISMATCH;
+	}
+	else
+		pkgdep->match = pkgconf_pkg_ref(client, pkg);
 
 	return pkg;
 }
@@ -1277,16 +1353,14 @@ pkgconf_pkg_verify_graph(pkgconf_client_t *client, pkgconf_pkg_t *root, int dept
 static unsigned int
 pkgconf_pkg_report_graph_error(pkgconf_client_t *client, pkgconf_pkg_t *parent, pkgconf_pkg_t *pkg, pkgconf_dependency_t *node, unsigned int eflags)
 {
-	static bool already_sent_notice = false;
-
 	if (eflags & PKGCONF_PKG_ERRF_PACKAGE_NOT_FOUND)
 	{
-		if (!(client->flags & PKGCONF_PKG_PKGF_SIMPLIFY_ERRORS) & !already_sent_notice)
+		if (!(client->flags & PKGCONF_PKG_PKGF_SIMPLIFY_ERRORS) & !client->already_sent_notice)
 		{
 			pkgconf_error(client, "Package %s was not found in the pkg-config search path.\n", node->package);
 			pkgconf_error(client, "Perhaps you should add the directory containing `%s.pc'\n", node->package);
 			pkgconf_error(client, "to the PKG_CONFIG_PATH environment variable\n");
-			already_sent_notice = true;
+			client->already_sent_notice = true;
 		}
 
 		pkgconf_error(client, "Package '%s', required by '%s', not found\n", node->package, parent->id);
@@ -1370,7 +1444,7 @@ pkgconf_pkg_walk_conflicts_list(pkgconf_client_t *client,
 		if (*parentnode->package == '\0')
 			continue;
 
-		PKGCONF_FOREACH_LIST_ENTRY(root->requires.head, childnode)
+		PKGCONF_FOREACH_LIST_ENTRY(root->required.head, childnode)
 		{
 			pkgconf_pkg_t *pkgdep;
 			pkgconf_dependency_t *depnode = childnode->data;
@@ -1384,8 +1458,12 @@ pkgconf_pkg_walk_conflicts_list(pkgconf_client_t *client,
 				pkgconf_error(client, "Version '%s' of '%s' conflicts with '%s' due to satisfying conflict rule '%s %s%s%s'.\n",
 					pkgdep->version, pkgdep->realname, root->realname, parentnode->package, pkgconf_pkg_get_comparator(parentnode),
 					parentnode->version != NULL ? " " : "", parentnode->version != NULL ? parentnode->version : "");
-				pkgconf_error(client, "It may be possible to ignore this conflict and continue, try the\n");
-				pkgconf_error(client, "PKG_CONFIG_IGNORE_CONFLICTS environment variable.\n");
+
+				if (!(client->flags & PKGCONF_PKG_PKGF_SIMPLIFY_ERRORS))
+				{
+					pkgconf_error(client, "It may be possible to ignore this conflict and continue, try the\n");
+					pkgconf_error(client, "PKG_CONFIG_IGNORE_CONFLICTS environment variable.\n");
+				}
 
 				pkgconf_pkg_unref(client, pkgdep);
 
@@ -1442,7 +1520,7 @@ pkgconf_pkg_traverse(pkgconf_client_t *client,
 	}
 
 	PKGCONF_TRACE(client, "%s: walking requires list", root->id);
-	eflags = pkgconf_pkg_walk_list(client, root, &root->requires, func, data, maxdepth);
+	eflags = pkgconf_pkg_walk_list(client, root, &root->required, func, data, maxdepth);
 	if (eflags != PKGCONF_PKG_ERRF_OK)
 		return eflags;
 
