@@ -2,7 +2,7 @@
  * main.c
  * main() routine, printer functions
  *
- * Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017
+ * Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
  *     pkgconf authors (see AUTHORS).
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -14,9 +14,15 @@
  * from the use of this software.
  */
 
+#include <libpkgconf/stdinc.h>
 #include <libpkgconf/libpkgconf.h>
-#include "config.h"
+#include "libpkgconf/config.h"
 #include "getopt_long.h"
+#include "renderer-msvc.h"
+#ifdef _WIN32
+#include <io.h>     /* for _setmode() */
+#include <fcntl.h>
+#endif
 
 #define PKG_CFLAGS_ONLY_I		(((uint64_t) 1) << 2)
 #define PKG_CFLAGS_ONLY_OTHER		(((uint64_t) 1) << 3)
@@ -58,8 +64,10 @@
 #define PKG_DEBUG			(((uint64_t) 1) << 38)
 #define PKG_SHORT_ERRORS		(((uint64_t) 1) << 39)
 #define PKG_EXISTS			(((uint64_t) 1) << 40)
+#define PKG_MSVC_SYNTAX			(((uint64_t) 1) << 41)
 
 static pkgconf_client_t pkg_client;
+static const pkgconf_fragment_render_ops_t *want_render_ops = NULL;
 
 static uint64_t want_flags;
 static int maximum_traverse_depth = 2000;
@@ -161,7 +169,7 @@ print_requires(pkgconf_pkg_t *pkg)
 {
 	pkgconf_node_t *node;
 
-	PKGCONF_FOREACH_LIST_ENTRY(pkg->requires.head, node)
+	PKGCONF_FOREACH_LIST_ENTRY(pkg->required.head, node)
 	{
 		pkgconf_dependency_t *dep = node->data;
 
@@ -214,18 +222,16 @@ static bool
 apply_provides(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
 		print_provides(pkg);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 
 	return true;
@@ -240,7 +246,7 @@ print_digraph_node(pkgconf_client_t *client, pkgconf_pkg_t *pkg, void *unused)
 
 	printf("\"%s\" [fontname=Sans fontsize=8]\n", pkg->id);
 
-	PKGCONF_FOREACH_LIST_ENTRY(pkg->requires.head, node)
+	PKGCONF_FOREACH_LIST_ENTRY(pkg->required.head, node)
 	{
 		pkgconf_dependency_t *dep = node->data;
 
@@ -270,19 +276,17 @@ static bool
 apply_modversion(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
 		if (pkg->version != NULL)
 			printf("%s\n", pkg->version);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 
 	return true;
@@ -292,18 +296,16 @@ static bool
 apply_variables(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
 		print_variables(pkg);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 
 	return true;
@@ -313,18 +315,18 @@ static bool
 apply_path(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
-		printf("%s\n", pkg->filename);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
+		/* a module entry with no filename is either virtual, static (builtin) or synthesized. */
+		if (pkg->filename != NULL)
+			printf("%s\n", pkg->filename);
 	}
 
 	return true;
@@ -346,19 +348,15 @@ apply_variable(pkgconf_client_t *client, pkgconf_pkg_t *world, void *variable, i
 	pkgconf_node_t *iter;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
-
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
+		pkgconf_pkg_t *pkg = dep->match;
 
 		if (iter->prev != NULL)
 			printf(" ");
 
 		print_variable(client, pkg, variable);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 
 	printf("\n");
@@ -366,6 +364,57 @@ apply_variable(pkgconf_client_t *client, pkgconf_pkg_t *world, void *variable, i
 	return true;
 }
 
+static bool
+apply_env_var(const char *prefix, pkgconf_client_t *client, pkgconf_pkg_t *world, int maxdepth,
+	unsigned int (*collect_fn)(pkgconf_client_t *client, pkgconf_pkg_t *world, pkgconf_list_t *list, int maxdepth),
+	bool (*filter_fn)(const pkgconf_client_t *client, const pkgconf_fragment_t *frag, void *data))
+{
+	pkgconf_list_t unfiltered_list = PKGCONF_LIST_INITIALIZER;
+	pkgconf_list_t filtered_list = PKGCONF_LIST_INITIALIZER;
+	unsigned int eflag;
+	char *render_buf;
+
+	eflag = collect_fn(client, world, &unfiltered_list, maxdepth);
+	if (eflag != PKGCONF_PKG_ERRF_OK)
+		return false;
+
+	pkgconf_fragment_filter(client, &filtered_list, &unfiltered_list, filter_fn, NULL);
+
+	if (filtered_list.head == NULL)
+		goto out;
+
+	render_buf = pkgconf_fragment_render(&filtered_list, true, want_render_ops);
+	printf("%s='%s'\n", prefix, render_buf);
+	free(render_buf);
+
+out:
+	pkgconf_fragment_free(&unfiltered_list);
+	pkgconf_fragment_free(&filtered_list);
+
+	return true;
+}
+
+static bool
+apply_env(pkgconf_client_t *client, pkgconf_pkg_t *world, void *env_prefix_p, int maxdepth)
+{
+	const char *want_env_prefix = env_prefix_p, *it;
+	char workbuf[PKGCONF_ITEM_SIZE];
+
+	for (it = want_env_prefix; *it != '\0'; it++)
+		if (!isalpha(*it) && !isdigit(*it))
+			return false;
+
+	snprintf(workbuf, sizeof workbuf, "%s_CFLAGS", want_env_prefix);
+	if (!apply_env_var(workbuf, client, world, maxdepth, pkgconf_pkg_cflags, filter_cflags))
+		return false;
+
+	snprintf(workbuf, sizeof workbuf, "%s_LIBS", want_env_prefix);
+	if (!apply_env_var(workbuf, client, world, maxdepth, pkgconf_pkg_libs, filter_libs))
+		return false;
+
+	return true;
+}
+
 static bool
 apply_cflags(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
@@ -382,12 +431,13 @@ apply_cflags(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int m
 	pkgconf_fragment_filter(client, &filtered_list, &unfiltered_list, filter_cflags, NULL);
 
 	if (filtered_list.head == NULL)
-		return true;
+		goto out;
 
-	render_buf = pkgconf_fragment_render(&filtered_list, true);
+	render_buf = pkgconf_fragment_render(&filtered_list, true, want_render_ops);
 	printf("%s", render_buf);
 	free(render_buf);
 
+out:
 	pkgconf_fragment_free(&unfiltered_list);
 	pkgconf_fragment_free(&filtered_list);
 
@@ -410,12 +460,13 @@ apply_libs(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int max
 	pkgconf_fragment_filter(client, &filtered_list, &unfiltered_list, filter_libs, NULL);
 
 	if (filtered_list.head == NULL)
-		return true;
+		goto out;
 
-	render_buf = pkgconf_fragment_render(&filtered_list, true);
+	render_buf = pkgconf_fragment_render(&filtered_list, true, want_render_ops);
 	printf("%s", render_buf);
 	free(render_buf);
 
+out:
 	pkgconf_fragment_free(&unfiltered_list);
 	pkgconf_fragment_free(&filtered_list);
 
@@ -426,18 +477,16 @@ static bool
 apply_requires(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
 		print_requires(pkg);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 
 	return true;
@@ -447,18 +496,16 @@ static bool
 apply_requires_private(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
 {
 	pkgconf_node_t *iter;
+	(void) client;
 	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->requires.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
 	{
-		pkgconf_pkg_t *pkg;
 		pkgconf_dependency_t *dep = iter->data;
+		pkgconf_pkg_t *pkg = dep->match;
 
-		pkg = pkgconf_pkg_verify_dependency(client, dep, NULL);
 		print_requires_private(pkg);
-
-		pkgconf_pkg_free(&pkg_client, pkg);
 	}
 	return true;
 }
@@ -499,7 +546,7 @@ print_graph_node(pkgconf_client_t *client, pkgconf_pkg_t *pkg, void *data)
 	if (pkg->version != NULL)
 		printf("    version = '%s';\n", pkg->version);
 
-	PKGCONF_FOREACH_LIST_ENTRY(pkg->requires.head, n)
+	PKGCONF_FOREACH_LIST_ENTRY(pkg->required.head, n)
 	{
 		pkgconf_dependency_t *dep = n->data;
 		printf("    dependency '%s'", dep->package);
@@ -540,7 +587,7 @@ static void
 about(void)
 {
 	printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
-	printf("Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017\n");
+	printf("Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018\n");
 	printf("    pkgconf authors (see AUTHORS in documentation directory).\n\n");
 	printf("Permission to use, copy, modify, and/or distribute this software for any\n");
 	printf("purpose with or without fee is hereby granted, provided that the above\n");
@@ -620,6 +667,8 @@ usage(void)
 	printf("  --keep-system-cflags              keep -I%s entries in cflags output\n", SYSTEM_INCLUDEDIR);
 	printf("  --keep-system-libs                keep -L%s entries in libs output\n", SYSTEM_LIBDIR);
 	printf("  --path                            show the exact filenames for any matching .pc files\n");
+	printf("  --modversion                      print the specified module's version to stdout\n");
+	printf("  --msvc-syntax                     print translatable fragments in MSVC syntax\n");
 
 	printf("\nreport bugs to <%s>.\n", PACKAGE_BUGREPORT);
 }
@@ -648,10 +697,21 @@ main(int argc, char *argv[])
 	char *required_max_module_version = NULL;
 	char *required_module_version = NULL;
 	char *logfile_arg = NULL;
+	char *want_env_prefix = NULL;
 	unsigned int want_client_flags = PKGCONF_PKG_PKGF_NONE;
 
 	want_flags = 0;
 
+#ifdef _WIN32
+	/* When running regression tests in cygwin, and building native
+	 * executable, tests fail unless native executable outputs unix
+	 * line endings.  Come to think of it, this will probably help
+	 * real people who use cygwin build environments but native pkgconf, too.
+	 */
+	_setmode(fileno(stdout), O_BINARY);
+	_setmode(fileno(stderr), O_BINARY);
+#endif
+
 	struct pkg_option options[] = {
 		{ "version", no_argument, &want_flags, PKG_VERSION|PKG_PRINT_ERRORS, },
 		{ "about", no_argument, &want_flags, PKG_ABOUT|PKG_PRINT_ERRORS, },
@@ -704,6 +764,8 @@ main(int argc, char *argv[])
 		{ "relocate", required_argument, NULL, 45 },
 		{ "dont-define-prefix", no_argument, &want_flags, PKG_DONT_DEFINE_PREFIX },
 		{ "dont-relocate-paths", no_argument, &want_flags, PKG_DONT_RELOCATE_PATHS },
+		{ "env", required_argument, NULL, 48 },
+		{ "msvc-syntax", no_argument, &want_flags, PKG_MSVC_SYNTAX },
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -752,6 +814,9 @@ main(int argc, char *argv[])
 		case 45:
 			relocate_path(pkg_optarg);
 			return EXIT_SUCCESS;
+		case 48:
+			want_env_prefix = pkg_optarg;
+			break;
 		case '?':
 		case ':':
 			return EXIT_FAILURE;
@@ -761,6 +826,9 @@ main(int argc, char *argv[])
 		}
 	}
 
+	if ((want_flags & PKG_MSVC_SYNTAX) == PKG_MSVC_SYNTAX)
+		want_render_ops = msvc_renderer_get();
+
 	if ((env_traverse_depth = getenv("PKG_CONFIG_MAXIMUM_TRAVERSE_DEPTH")) != NULL)
 		maximum_traverse_depth = atoi(env_traverse_depth);
 
@@ -1087,6 +1155,17 @@ main(int argc, char *argv[])
 		goto out;
 	}
 
+	if (want_env_prefix != NULL)
+	{
+		if (!pkgconf_queue_apply(&pkg_client, &pkgq, apply_env, maximum_traverse_depth, want_env_prefix))
+		{
+			ret = EXIT_FAILURE;
+			goto out;
+		}
+
+		want_flags = 0;
+	}
+
 	if ((want_flags & PKG_PROVIDES) == PKG_PROVIDES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
